import {FormControl, FormGroup} from '@angular/forms';

export class Util {

  /**
   * 数组排序
   * @param arr 排序的数组对象
   * @param condition 依据排序的字段
   * @param isAsc 是否升序排列
   */
  public static arraySort(arr: any[], condition: string, isAsc?: boolean): any[] {
    arr.sort((a, b) => {
      if (a[condition] > b[condition]) {
        if (isAsc) {
          return -1;
        } else {
          return 1;
        }
      }
      if (a[condition] < b[condition]) {
        if (isAsc) {
          return 1;
        } else {
          return -1;
        }
      }
      return 0;
    });
    return arr;
  }

  /**
   * 深度转化json字符串为对象
   * @param str 需序列化的json字符串
   */
  public static parseJson(str: string): object | string {
    if (str.charAt(0) === '{') {
      try {
        const obj = JSON.parse(str);
        for (const key in obj) {
          if (obj.hasOwnProperty(key) && typeof obj[key] === 'string') {
            obj[key] = Util.parseJson(obj[key]);
          }
        }
        return obj;
      } catch (e) {
        return str;
      }
    } else {
      return str;
    }
  }

  /**
   * 深拷贝对象
   * @param data 需拷贝的对象
   * @param exclude 不需要拷贝的字段
   */
  public static deepClone(data: any, ...exclude: string[]): any {
    let o;
    let i;
    if (data instanceof Array) {
      o = [];
      for (i = 0; i < data.length; i++) {
        o.push(Util.deepClone(data[i], ...exclude));
      }
      return o;
    } else if (data === null) {
      return data;
    } else if (typeof data === 'object') {
      o = {};
      for (i in data) {
        if (data.hasOwnProperty(i) && !exclude?.includes(i)) {
          // @ts-ignore
          o[i] = Util.deepClone(data[i], ...exclude);
        }
      }
      return o;
    } else {
      return data;
    }
  }

  public static uuid(): string {
    function S4() {
      // tslint:disable-next-line:no-bitwise
      return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    }

    return (S4() + S4() + '-' + S4() + '-' + S4() + '-' + S4() + '-' + S4() + S4() + S4());
  }

  public static round(num: number, count: number = 2): number {
    if (!num) {
      return 0;
    }
    return Math.round(num * (10 ** count)) / (10 ** count);
  }

  public static parseNumber(value: string) {
    if (!value) {
      return 0;
    }
    return isNaN(Number.parseFloat(value)) ? 0 : Number.parseFloat(value);
  }

  public static compareValue(value1: any, value2: any): 1 | -1 | 0 {
    switch (typeof value1) {
      case 'string':
        return value1.localeCompare(value2, 'zh') as 1 | -1 | 0;
      case 'number':
        if (value1 !== value2) {
          return value1 < value2 ? -1 : 1;
        }
        break;
      case 'object':
        if (value1 instanceof Date) {
          const d1 = value1 && value1.getTime();
          const d2 = value2 && value2.getTime();
          if (d1 !== d2) {
            return d1 < d2 ? -1 : 1;
          }
          break;
        }
    }
    return 0;
  }

  public static getSortBy(sortBys?: 'asc' | 'desc' | Array<'asc' | 'desc'>, index = 0): 'asc' | 'desc' {
    if (sortBys) {
      if (sortBys instanceof Array) {
        return sortBys[index] || sortBys[sortBys.length - 1];
      } else {
        return sortBys;
      }
    }
    return 'asc';
  }

  public static getObjProperty(property: string, obj: any): any {
    const propertyArr = property.split('.');
    obj = obj[propertyArr.splice(0, 1).toString()];
    if (propertyArr.length) {
      return Util.getObjProperty(propertyArr.join('.'), obj);
    }
    return obj;
  }

  public static setObjProperty(property: string, obj: any, value: any): void {
    const propertyArr = property.split('.');
    if (propertyArr.length > 1) {
      obj = obj[propertyArr.splice(0, 1).toString()];
      Util.setObjProperty(propertyArr.join('.'), obj, value);
    } else {
      obj[propertyArr[0]] = value;
    }
  }

  public static markControlAsDirtyWithChildren(formGroup: FormGroup) {
    for (const controlName in formGroup.controls) {
      if (formGroup.controls.hasOwnProperty(controlName)) {
        const control = formGroup.get(controlName);
        if (control instanceof FormControl) {
          control.markAsDirty();
          control.updateValueAndValidity();
        } else if (control instanceof FormGroup) {
          Util.markControlAsDirtyWithChildren(control);
        }
      }
    }
  }

  public static cleanProperty(data: any): any {
    const tmp = JSON.parse(JSON.stringify(data));
    if (tmp.__property) {
      delete tmp.__property;
      for (const key in tmp) {
        if (tmp.hasOwnProperty(key)) {
          if (tmp[key] === null || tmp[key] === undefined) {
            continue;
          }
          if (tmp[key].__property) {
            tmp[key] = Util.cleanProperty(tmp[key]);
          }
          if (tmp[key] instanceof Array) {
            for (let i = 0; i < tmp[key].length; i++) {
              tmp[key][i] = Util.cleanProperty(tmp[key][i]);
            }
          }
        }
      }
    }
    return tmp;
  }

  public static addUrlParam(url: string, param: string): string {
    return url + ((url.indexOf('?') >= 0) ? '&' : '?') + param;
  }

  public static transformDoubleTypeToSingleType(data: string): string {
    if (!data) {
      return '';
    }
    const regxForLeftParentheses = new RegExp('（', 'g');
    const regxForRightParentheses = new RegExp('）', 'g');
    const regxForColon = new RegExp('：', 'g');
    const regxForComma = new RegExp('，', 'g');
    const regxForQuotes = new RegExp(/[“”‘’]/, 'g');
    return data.replace(regxForLeftParentheses, '(')
      .replace(regxForRightParentheses, ')')
      .replace(regxForColon, ':')
      .replace(regxForComma, ',')
      .replace(regxForQuotes, '"');
  }

  public static downloadFile(url: string): void {
    const a = document.createElement('a');
    a.href = url;
    a.setAttribute('download', '');
    a.click();
  }
}
